# This is an R script template for the use of Applied Microeconometrics I, Fall 2023 
# This script can be used as a problem set template
# Script created by Ramin Izadi


# Always start R with a clean evironment to avoid conflicts ------------
# To execute code line by line, use the hotkey ctrl+enter
# You can also highlight any part of the code and execute it with ctrl+enter
# A hashtag (#) at the beginning of a line starts a comment that is excluded from the code. Similar to * in STATA.
rm(list = ls()) # corresponds to 'clear all' in STATA


# Setting your working directory ------------
# Click Session -> Set Working Directory -> Choose Directory
# Copy  the console output and paste below

setwd("C:/Users/izadir1/Google Drive/PhD/Teaching assistant/2018 Labor Economics/R tutorial")
getwd()

# Now you can use relative file paths to read and save data (shown below with read.dta)! 


# Install and load all packages that you will need ------------

# This will install user created content that come in 'packages'. Install is only needed once.
#install.packages('foreign')
#install.packages('sandwich')

# This will load that content into the R environment (usually functions)
library(foreign) # for read.dta function
library(tidyverse)
library(sandwich)
library(lmtest) # for coeftest function


# Reading data to R ------------
# Two kinds of R files like in stata: .Rdata for data, .R for scripts.
# R stores information in objects. They can be functions, data frames, vectors, matrices, models.
# Using relative file paths, the dot refers to your current working directory.

?read.dta # Use '?' to get help about a function. Similar to 'help' in STATA.
countries = read.dta('./data/repl_master_stats.dta')


# Subsetting data frames ---------

# '[' is a special function for subsetting. You will be using this a lot in R.
# Subsetting works like this for vectors: my_vector[index]
# Subsetting works like this for data frames and matrices: my_data[row_index, column_index]
# Subsetting works like this for lists: my_data[[list_index]]
# The *_index can be any of the following: 
##vector of row/column numbers,
##vector of row/column names, 
##logical vector of the same length as the dimension


# Lets create some vector objects first
important_variables = c("country", 
                        "countryname", 
                        "coreDummy", 
                        "refyear", 
                        "ln_gdppc_e9ry", 
                        "er_a_all", 
                        "hwp_a_all",
                        "q_e9ry")
first_rows = 1:5

# These are equivalent ways of subsetting the data
countries[first_rows, important_variables] #column names
countries[1:5, c("country", 
                 "countryname", 
                 "coreDummy", 
                 "refyear", 
                 "ln_gdppc_e9ry", 
                 "er_a_all", 
                 "hwp_a_all",
                 "q_e9ry")] #column names
countries[1:5, c(1, 2, 3, 5, 9, 10, 22)] #column number

# Let's use the vector of names to reduce the number of variables in our data and make it a little more accessible
countries = countries[ , important_variables]

# How to drop the row with your birthday?
countries = countries[-28, ]

# Replacing values
countries$refyear == 2010
sum(countries$refyear == 2010)
countries[countries$refyear == 2010, 'countryname']
countries[countries$refyear == 2010, 'refyear'] = 2011
countries[countries$refyear == 2010, 'countryname'] = "Finland"

# Saving the modified object
save(countries, file = './data_output/modified_repl_master_stats.Rdata')
# Load an R object
load('./data_output/modified_repl_master_stats.Rdata')


# Data summary and descriptives ------------
# Everything works with functions in R. Functions take arguments and turn them into output.
# You can recognize functions because they are always followed by brackets: function_name()
# The arguments of a function go inside the brackets. Just as you are used to in maths!

head(countries, 3)

str(countries) # similar to 'describe' in STATA

table(countries$coreDummy, useNA = 'always')
table(countries$refyear, useNA = 'always')
table(countries$refyear, countries$coreDummy, useNA = 'ifany') # similar to 'tabulate' in STATA

summary(countries)
summary(countries$hwp_a_all)

# by() applies a function to each subgroup of a variable
by(data = countries$hwp_a_all, INDICES = countries$q_e9ry, FUN = mean)


# Replicating figure 1 ---------

core = countries[countries$coreDummy == 1, ]
Hours_per_week = core[, "hwp_a_all"]
ln_GDP = core[, "ln_gdppc_e9ry"]

# plot() sets up the basic layout of the graph to which layers can be added wit subsequent commands
# such as text() and abline() used below.
plot(ln_GDP, Hours_per_week, ylim = c(0, 50))
text(ln_GDP,  Hours_per_week, labels = core$country, cex= 0.7, pos = 4)

# Add a linear fit
# Save the OLS regression results in an object called 'fit'.
fit = lm(hwp_a_all ~ ln_gdppc_e9ry, data = core)
summary(fit) # Homoscedastic standard errors
coeftest(fit, vcov = sandwich) # Heteroscedastic robust standard errors ('robust' option in STATA)
abline(fit) #adds the regression line to an EXISTING plot

# Doing the same with tidyverse
countries %>% 
  filter(coreDummy == 1) %>% 
  ggplot(aes(ln_gdppc_e9ry, hwp_a_all)) + 
  geom_point(aes(color = as.factor(q_e9ry))) +
  geom_text(aes(color = as.factor(q_e9ry), label = country), hjust=-0.2, vjust=0.5) +
  geom_smooth(method = "lm")


# Replicating table 1 ---------

# Regressing on group dummies is the same thing as taking conditional means:
table1_all = lm(hwp_a_all ~ as.factor(q_e9ry), data = countries)
table1_all
table1_core = lm(hwp_a_all ~ as.factor(q_e9ry), data = core)
table1_core

# Create a matrix
table1 = matrix(c(table1_core$coefficients, table1_all$coefficients), ncol = 3, byrow = T)
table1
# Name rows and columns
colnames(table1) = c("Low", "Middle", "High")
table1
rownames(table1) = c("Core countries", "All countries")
table1
# Convert to conditional means 
table1[, 2] = table1[, 1] + table1[, 2]
table1
table1[, 3] = table1[, 1] + table1[, 3]
table1


# Tips for Problem Set 3, Applied Microeconometrics I, Fall 2023 ----------

#install.packages("ivreg")
library(ivreg)
my_data = read.dta("JPE - Di Tella and Schargrodsky - CriminalRecidivismAfterPrisonAndElectronicMonitoring .dta")
# Read table 5 notes from the paper carefully
my_iv = ivreg(recidivism ~ electronicMonitoring | percJudgeSentToEM, # add covariates to both sides of | 
              data = my_data[my_data$offendersPerJudge >= 10, ])
summary(my_iv)
